;--------------------------------
;NSIS OpenLibraries Installation Script
;--------------------------------

!ifndef VERSION
  !define VERSION 0_5_0
  !define VERSION_DOT 0.5.0
!endif

;--------------------------------
;Configuration

!ifdef OUTFILE
  OutFile "${OUTFILE}"
!else
  OutFile openlibraries-${VERSION_DOT}-sdk.exe
!endif

SetCompressor /SOLID lzma

InstType "Full"

InstallDir $PROGRAMFILES\OpenLibraries
InstallDirRegKey HKLM Software\OpenLibraries ""

;--------------------------------
;Header Files

!define ALL_USERS

!include "MUI.nsh"
!include "Sections.nsh"
!include "add_to_path.nsh"
!include "write_env_str.nsh"

;Names
Name "OpenLibraries"
Caption "OpenLibraries (v.${VERSION_DOT}) Setup"

;Interface Settings
!define MUI_ABORTWARNING

!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_BITMAP "openlibraries.bmp" ; optional
!define MUI_WELCOMEFINISHPAGE_BITMAP "${NSISDIR}\Contrib\Graphics\Wizard\orange.bmp"

!define MUI_COMPONENTSPAGE_SMALLDESC

;Pages
!define MUI_WELCOMEPAGE_TITLE "Welcome to the OpenLibraries (v.${VERSION_DOT}) Setup Wizard"
!define MUI_WELCOMEPAGE_TEXT "This wizard will guide you through the installation of OpenLibraries.\r\n\r\n$_CLICK"

!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_LICENSE "..\COPYING"
;!insertmacro MUI_PAGE_COMPONENTS
;!insertmacro MUI_PAGE_DIRECTORY
!insertmacro MUI_PAGE_INSTFILES

!define MUI_FINISHPAGE_LINK "Visit the web site for the latest news, FAQs and support"
!define MUI_FINISHPAGE_LINK_LOCATION "http://www.openlibraries.org/"

!insertmacro MUI_PAGE_FINISH

!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES

;--------------------------------
;Languages

!insertmacro MUI_LANGUAGE "English"

 Function .onInit
 
  ReadRegStr $R0 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\OpenLibraries" "UninstallString"
  StrCmp $R0 "" 0 +2
  Return

  ;IfSilent 0 +6
  ;ReadRegStr $R1 HKLM "Software\OpenLibraries" "Version"
  ;StrCmp $R1 "${VERSION}" 0 +4
  ;ReadRegStr $R2 HKLM "Software\OpenLibraries" "Installation type"
  ;StrCmp $R2 "SDK" 0 +2
  ;Abort

  ;MessageBox MB_OKCANCEL|MB_ICONEXCLAMATION
  ;"OpenLibraries v${VERSION} is already installed. $\n$\nClick `OK` to remove the
  ;previous version or `Cancel` to cancel this upgrade."
  ;/SD IDOK IDOK uninst
  ;Abort
  
  ;Run the uninstaller
  ;uninst:
  ClearErrors
	  
  ;Start the uninstaller using the option to not copy itself
  ExecWait '$R0 /S _?=$INSTDIR'
	 
  IfErrors no_remove_uninstaller
  ; In most cases the uninstall is successful at this point.
  ; You may also consider using a registry key to check whether 
  ; the user has chosen to uninstall. If you are using an uninstaller
  ; components page, make sure all sections are uninstalled.
  Return
	    
  no_remove_uninstaller:
  MessageBox MB_ICONEXCLAMATION \
  "Unable to remove previous version of OpenLibraries"
  Abort
 
FunctionEnd

Section "OpenLibraries" SecCore

  SetDetailsPrint textonly
  DetailPrint "Installing OpenLibraries..."
  SetDetailsPrint listonly

  SectionIn 1 2 3 RO
  SetOutPath $INSTDIR

  SetOverwrite on

  SetOutPath $INSTDIR\include\openlibraries-${VERSION_DOT}\openpluginlib\pl
  File ..\src\openpluginlib\pl\allocatorstringstorage.h
  File ..\src\openpluginlib\pl\bind_info.hpp
  File ..\src\openpluginlib\pl\cg.hpp
  File ..\src\openpluginlib\pl\config.hpp
  File ..\src\openpluginlib\pl\content_handler_msxml.hpp
  File ..\src\openpluginlib\pl\cowstringopt.h
  File ..\src\openpluginlib\pl\discovery_traits.hpp
  File ..\src\openpluginlib\pl\flex_string.h
  File ..\src\openpluginlib\pl\flex_string_details.h
  File ..\src\openpluginlib\pl\flex_string_shell.h
  File ..\src\openpluginlib\pl\geometry.hpp
  File ..\src\openpluginlib\pl\GL_utility.hpp
  File ..\src\openpluginlib\pl\openplugin.hpp
  File ..\src\openpluginlib\pl\openpluginlib.hpp
  File ..\src\openpluginlib\pl\pool.hpp
  File ..\src\openpluginlib\pl\property.hpp
  File ..\src\openpluginlib\pl\registry.hpp
  File ..\src\openpluginlib\pl\shader_manager.hpp
  File ..\src\openpluginlib\pl\simplestringstorage.h
  File ..\src\openpluginlib\pl\smallstringopt.h
  File ..\src\openpluginlib\pl\stream.hpp
  File ..\src\openpluginlib\pl\string.hpp
  File ..\src\openpluginlib\pl\timer.hpp
  File ..\src\openpluginlib\pl\utf8_utils.hpp
  File ..\src\openpluginlib\pl\vectorstringstorage.h
  
  SetOutPath $INSTDIR\include\openlibraries-${VERSION_DOT}\openpluginlib\pl\pcos
  File ..\src\openpluginlib\pl\pcos\iproperty_container.hpp
  File ..\src\openpluginlib\pl\pcos\isubject.hpp
  File ..\src\openpluginlib\pl\pcos\iclonable.hpp
  File ..\src\openpluginlib\pl\pcos\key.hpp
  File ..\src\openpluginlib\pl\pcos\observer.hpp
  File ..\src\openpluginlib\pl\pcos\property.hpp
  File ..\src\openpluginlib\pl\pcos\property_container.hpp
  File ..\src\openpluginlib\pl\pcos\subject.hpp
  File ..\src\openpluginlib\pl\pcos\visitor.hpp
  
  SetOutPath $INSTDIR\include\openlibraries-${VERSION_DOT}\openimagelib\il
  File ..\src\openimagelib\il\basic_image.hpp
  File ..\src\openimagelib\il\compression_traits.hpp
  File ..\src\openimagelib\il\config.hpp
  File ..\src\openimagelib\il\float_traits.hpp
  File ..\src\openimagelib\il\il.hpp
  File ..\src\openimagelib\il\openimagelib_plugin.hpp
  File ..\src\openimagelib\il\rgb_traits.hpp
  File ..\src\openimagelib\il\traits.hpp
  File ..\src\openimagelib\il\utility.hpp
  File ..\src\openimagelib\il\yuv_traits.hpp
  
  SetOutPath $INSTDIR\include\openlibraries-${VERSION_DOT}\openmedialib\ml
  File ..\src\openmedialib\ml\audio.hpp
  File ..\src\openmedialib\ml\config.hpp
  File ..\src\openmedialib\ml\filter.hpp
  File ..\src\openmedialib\ml\frame.hpp
  File ..\src\openmedialib\ml\input.hpp
  File ..\src\openmedialib\ml\ml.hpp
  File ..\src\openmedialib\ml\openmedialib_plugin.hpp
  File ..\src\openmedialib\ml\store.hpp
  File ..\src\openmedialib\ml\utilities.hpp
 
  SetOutPath $INSTDIR\include\openlibraries-${VERSION_DOT}\openobjectlib\sg
  File ..\src\openobjectlib\sg\appearance.hpp
  File ..\src\openobjectlib\sg\bbox_volume.hpp
  File ..\src\openobjectlib\sg\calculate_tangent_space.hpp
  File ..\src\openobjectlib\sg\config.hpp
  File ..\src\openobjectlib\sg\constant_time_dispatcher.hpp
  File ..\src\openobjectlib\sg\core.hpp
  File ..\src\openobjectlib\sg\graph_updater.hpp
  File ..\src\openobjectlib\sg\hw_GL_renderer.hpp
  File ..\src\openobjectlib\sg\lexical_cast.hpp
  File ..\src\openobjectlib\sg\meshes.hpp
  File ..\src\openobjectlib\sg\openobjectlib_plugin.hpp
  File ..\src\openobjectlib\sg\render_buffer.hpp
  File ..\src\openobjectlib\sg\scene.hpp
  File ..\src\openobjectlib\sg\sg.hpp
  File ..\src\openobjectlib\sg\shaders.hpp
  File ..\src\openobjectlib\sg\spatial.hpp
  File ..\src\openobjectlib\sg\spirit.hpp

  SetOutPath $INSTDIR\include\openlibraries-${VERSION_DOT}\openassetlib\al
  File ..\src\openassetlib\al\al.hpp
  File ..\src\openassetlib\al\al_plugin.hpp
  File ..\src\openassetlib\al\asset.hpp
  File ..\src\openassetlib\al\base64_codec.hpp
  File ..\src\openassetlib\al\config.hpp
  File ..\src\openassetlib\al\guid.hpp
  File ..\src\openassetlib\al\set.hpp

  SetOutPath $INSTDIR\bin
  File "..\src\openpluginlib\pl\Multi-threaded Debug DLL\openpluginlib_pl-vc80-d-${VERSION}.dll"
  File "..\src\openpluginlib\pl\Multi-threaded Release DLL\openpluginlib_pl-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\il\Multi-threaded Debug DLL\openimagelib_il-vc80-d-${VERSION}.dll"
  File "..\src\openimagelib\il\Multi-threaded Release DLL\openimagelib_il-vc80-r-${VERSION}.dll"
  File "..\src\openmedialib\ml\Multi-threaded Debug DLL\openmedialib_ml-vc80-d-${VERSION}.dll"
  File "..\src\openmedialib\ml\Multi-threaded Release DLL\openmedialib_ml-vc80-r-${VERSION}.dll"
  File "..\src\openobjectlib\sg\Multi-threaded Debug DLL\openobjectlib_sg-vc80-d-${VERSION}.dll"
  File "..\src\openobjectlib\sg\Multi-threaded Release DLL\openobjectlib_sg-vc80-r-${VERSION}.dll"
  File "..\src\openassetlib\al\Multi-threaded Debug DLL\openassetlib_al-vc80-d-${VERSION}.dll"
  File "..\src\openassetlib\al\Multi-threaded Release DLL\openassetlib_al-vc80-r-${VERSION}.dll"

  ; dependencies
  File "C:\Program Files\FFmpeg\lib\avcodec-51.dll"
  File "C:\Program Files\FFmpeg\lib\avformat-52.dll"
  File "C:\Program Files\FFmpeg\lib\avutil-49.dll"
  File "C:\Program Files\FFmpeg\lib\libmp3lame-0.dll"
;  File "C:\Program Files\FFmpeg\lib\libogg-0.dll"
  File "C:\OpenEXR\bin\Debug\Half.dll"
  File "C:\OpenEXR\bin\Debug\Iex.dll"
  File "C:\OpenEXR\bin\Debug\IlmImf.dll"
  File "C:\OpenEXR\bin\Debug\Half.dll"
  File "C:\OpenEXR\bin\Debug\IlmThread.dll"
  File "C:\OpenEXR\bin\Debug\Imath.dll"
;  File "C:\SDL-1.2.12\lib\SDL.dll"
  File "C:\Program Files\NVIDIA Corporation\Cg\bin\cg.dll"
  File "C:\Program Files\NVIDIA Corporation\Cg\bin\cgGL.dll"
  File "C:\Windows\system32\msvcp71.dll"
  File "C:\Windows\system32\msvcr71.dll"
  File "C:\Windows\system32\glew32.dll"
  File "c:\Windows\system32\python25.dll"
  File "C:\Qt\3.3.8\bin\qt-mt338.dll"
  File "c:\Boost\lib\boost_python-vc80-mt-1_34_1.dll"
  File "C:\zlib\lib\zlib1.dll"

  ; Python
  SetOutPath $INSTDIR\python\release
  File "..\src\openpluginlib\py\Multi-threaded Release DLL\openpluginlib.pyd"
  File "..\src\openimagelib\py\Multi-threaded Release DLL\openimagelib.pyd"
  File "..\src\openmedialib\py\Multi-threaded Release DLL\openmedialib.pyd"
  File "..\src\openassetlib\py\Multi-threaded Release DLL\openassetlib.pyd"

  ; Python
  SetOutPath $INSTDIR\python\debug
  File "..\src\openpluginlib\py\Multi-threaded Debug DLL\openpluginlib.pyd"
  File "..\src\openimagelib\py\Multi-threaded Debug DLL\openimagelib.pyd"
  File "..\src\openmedialib\py\Multi-threaded Debug DLL\openmedialib.pyd"
  File "..\src\openassetlib\py\Multi-threaded Debug DLL\openassetlib.pyd"
  
  SetOutPath $INSTDIR\lib
  File "..\src\openpluginlib\pl\Multi-threaded Debug DLL\openpluginlib_pl-vc80-d-${VERSION}.lib"
  File "..\src\openpluginlib\pl\Multi-threaded Release DLL\openpluginlib_pl-vc80-r-${VERSION}.lib"
  File "..\src\openimagelib\il\Multi-threaded Debug DLL\openimagelib_il-vc80-d-${VERSION}.lib"
  File "..\src\openimagelib\il\Multi-threaded Release DLL\openimagelib_il-vc80-r-${VERSION}.lib"
  File "..\src\openmedialib\ml\Multi-threaded Debug DLL\openmedialib_ml-vc80-d-${VERSION}.lib"
  File "..\src\openmedialib\ml\Multi-threaded Release DLL\openmedialib_ml-vc80-r-${VERSION}.lib"
  File "..\src\openobjectlib\sg\Multi-threaded Debug DLL\openobjectlib_sg-vc80-d-${VERSION}.lib"
  File "..\src\openobjectlib\sg\Multi-threaded Release DLL\openobjectlib_sg-vc80-r-${VERSION}.lib"
  File "..\src\openassetlib\al\Multi-threaded Debug DLL\openassetlib_al-vc80-d-${VERSION}.lib"
  File "..\src\openassetlib\al\Multi-threaded Release DLL\openassetlib_al-vc80-r-${VERSION}.lib"
   
  SetOutPath $INSTDIR\plugins\debug
  File "..\src\openimagelib\plugins\3D_lightmap\3D_lightmap_plugin.opl"
  File "..\src\openimagelib\plugins\3D_lightmap\Multi-threaded Debug DLL\openimagelib_3D_lightmap-vc80-d-${VERSION}.dll"
  File "..\src\openimagelib\plugins\dds\dds_plugin.opl"
  File "..\src\openimagelib\plugins\dds\Multi-threaded Debug DLL\openimagelib_dds-vc80-d-${VERSION}.dll"
  File "..\src\openimagelib\plugins\dpx\dpx_plugin.opl"
  File "..\src\openimagelib\plugins\dpx\Multi-threaded Debug DLL\openimagelib_dpx-vc80-d-${VERSION}.dll"
  File "..\src\openimagelib\plugins\exr\exr_plugin.opl"
  File "..\src\openimagelib\plugins\exr\Multi-threaded Debug DLL\openimagelib_exr-vc80-d-${VERSION}.dll"
  File "..\src\openimagelib\plugins\gdi+\gdi_plugin.opl"
  File "..\src\openimagelib\plugins\gdi+\Multi-threaded Debug DLL\openimagelib_gdi-vc80-d-${VERSION}.dll"
  File "..\src\openimagelib\plugins\hdr\hdr_plugin.opl"
  File "..\src\openimagelib\plugins\hdr\Multi-threaded Debug DLL\openimagelib_hdr-vc80-d-${VERSION}.dll"
  File "..\src\openimagelib\plugins\sgi\sgi_plugin.opl"
  File "..\src\openimagelib\plugins\sgi\Multi-threaded Debug DLL\openimagelib_sgi-vc80-d-${VERSION}.dll"
  File "..\src\openimagelib\plugins\tga\tga_plugin.opl"
  File "..\src\openimagelib\plugins\tga\Multi-threaded Debug DLL\openimagelib_tga-vc80-d-${VERSION}.dll"
  File "..\src\openmedialib\plugins\avformat\avformat_plugin.opl"
  File "..\src\openmedialib\plugins\avformat\Multi-threaded Debug DLL\openmedialib_avformat-vc80-d-${VERSION}.dll"
  File "..\src\openmedialib\plugins\oil\oil_plugin.opl"
  File "..\src\openmedialib\plugins\oil\Multi-threaded Debug DLL\openmedialib_oil-vc80-d-${VERSION}.dll"
  File "..\src\openmedialib\plugins\openal\openal_plugin.opl"
  File "..\src\openmedialib\plugins\openal\Multi-threaded Debug DLL\openmedialib_openal-vc80-d-${VERSION}.dll"
  File "..\src\openmedialib\plugins\gensys\gensys_plugin.opl"
  File "..\src\openmedialib\plugins\gensys\Multi-threaded Debug DLL\openmedialib_gensys-vc80-d-${VERSION}.dll"
  File "..\src\openmedialib\plugins\quicktime\quicktime_plugin.opl"
  File "..\src\openmedialib\plugins\quicktime\Multi-threaded Debug DLL\openmedialib_quicktime-vc80-d-${VERSION}.dll"
;  File "..\src\openmedialib\plugins\sdl\sdl_plugin.opl"
;  File "..\src\openmedialib\plugins\sdl\Multi-threaded Debug DLL\openmedialib_sdl-vc80-d-${VERSION}.dll"
  File "..\src\openobjectlib\plugins\wavefront\obj_plugin.opl"
  File "..\src\openobjectlib\plugins\wavefront\Multi-threaded Debug DLL\openobjectlib_obj-vc80-d-${VERSION}.dll"
  File "..\src\openobjectlib\plugins\X3D\x3d_plugin.opl"
  File "..\src\openobjectlib\plugins\X3D\Multi-threaded Debug DLL\openobjectlib_x3d-vc80-d-${VERSION}.dll"
;  File "..\src\openobjectlib\plugins\Collada\collada_plugin.opl"
;  File "..\src\openobjectlib\plugins\Collada\Multi-threaded Debug DLL\openobjectlib_collada-vc80-d-${VERSION}.dll"
  File "..\src\openeffectslib\plugins\tonemap\tonemap_plugin.opl"
  File "..\src\openeffectslib\plugins\tonemap\Multi-threaded Debug DLL\openeffectslib_tonemap-vc80-d-${VERSION}.dll"
  File "..\src\openassetlib\plugins\sqlite\sqlite3_metadata_plugin.opl"
  File "..\src\openassetlib\plugins\sqlite\Multi-threaded Debug DLL\openassetlib_sqlite-vc80-d-${VERSION}.dll"
  
  SetOutPath $INSTDIR\plugins\release
  File "..\src\openimagelib\plugins\3D_lightmap\3D_lightmap_plugin.opl"
  File "..\src\openimagelib\plugins\3D_lightmap\Multi-threaded Release DLL\openimagelib_3D_lightmap-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\dds\dds_plugin.opl"
  File "..\src\openimagelib\plugins\dds\Multi-threaded Release DLL\openimagelib_dds-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\dpx\dpx_plugin.opl"
  File "..\src\openimagelib\plugins\dpx\Multi-threaded Release DLL\openimagelib_dpx-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\exr\exr_plugin.opl"
  File "..\src\openimagelib\plugins\exr\Multi-threaded Release DLL\openimagelib_exr-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\gdi+\gdi_plugin.opl"
  File "..\src\openimagelib\plugins\gdi+\Multi-threaded Release DLL\openimagelib_gdi-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\hdr\hdr_plugin.opl"
  File "..\src\openimagelib\plugins\hdr\Multi-threaded Release DLL\openimagelib_hdr-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\sgi\sgi_plugin.opl"
  File "..\src\openimagelib\plugins\sgi\Multi-threaded Release DLL\openimagelib_sgi-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\tga\tga_plugin.opl"
  File "..\src\openimagelib\plugins\tga\Multi-threaded Release DLL\openimagelib_tga-vc80-r-${VERSION}.dll"
  File "..\src\openmedialib\plugins\avformat\avformat_plugin.opl"
  File "..\src\openmedialib\plugins\avformat\Multi-threaded Release DLL\openmedialib_avformat-vc80-r-${VERSION}.dll"
  ;File "..\src\openmedialib\plugins\dshow\dshow_plugin.opl"
  ;File "..\src\openmedialib\plugins\dshow\Multi-threaded Release DLL\openmedialib_dshow-vc80-r-${VERSION}.dll"
  File "..\src\openmedialib\plugins\oil\oil_plugin.opl"
  File "..\src\openmedialib\plugins\oil\Multi-threaded Release DLL\openmedialib_oil-vc80-r-${VERSION}.dll"
  File "..\src\openmedialib\plugins\openal\openal_plugin.opl"
  File "..\src\openmedialib\plugins\openal\Multi-threaded Release DLL\openmedialib_openal-vc80-r-${VERSION}.dll"
  File "..\src\openmedialib\plugins\gensys\gensys_plugin.opl"
  File "..\src\openmedialib\plugins\gensys\Multi-threaded Release DLL\openmedialib_gensys-vc80-r-${VERSION}.dll"
  File "..\src\openmedialib\plugins\quicktime\quicktime_plugin.opl"
  File "..\src\openmedialib\plugins\quicktime\Multi-threaded Release DLL\openmedialib_quicktime-vc80-r-${VERSION}.dll"
;  File "..\src\openmedialib\plugins\sdl\sdl_plugin.opl"
;  File "..\src\openmedialib\plugins\sdl\Multi-threaded Release DLL\openmedialib_sdl-vc80-r-${VERSION}.dll"
  File "..\src\openobjectlib\plugins\wavefront\obj_plugin.opl"
  File "..\src\openobjectlib\plugins\wavefront\Multi-threaded Release DLL\openobjectlib_obj-vc80-r-${VERSION}.dll"
  File "..\src\openobjectlib\plugins\X3D\x3d_plugin.opl"
  File "..\src\openobjectlib\plugins\X3D\Multi-threaded Release DLL\openobjectlib_x3d-vc80-r-${VERSION}.dll"
;  File "..\src\openobjectlib\plugins\Collada\collada_plugin.opl"
;  File "..\src\openobjectlib\plugins\Collada\Multi-threaded Release DLL\openobjectlib_collada-vc80-r-${VERSION}.dll"
  File "..\src\openeffectslib\plugins\tonemap\tonemap_plugin.opl"
  File "..\src\openeffectslib\plugins\tonemap\Multi-threaded Release DLL\openeffectslib_tonemap-vc80-r-${VERSION}.dll"
  File "..\src\openassetlib\plugins\sqlite\sqlite3_metadata_plugin.opl"
  File "..\src\openassetlib\plugins\sqlite\Multi-threaded Release DLL\openassetlib_sqlite-vc80-r-${VERSION}.dll"
    
  SetOutPath $INSTDIR\shaders
  File "..\effects\shaders\sepia.cg"
  File "..\effects\shaders\sepia.frag"
  File "..\effects\shaders\fixed_function.vert"
  File "..\effects\shaders\passthrough.vert"
  File "..\effects\shaders\passthrough.cg"
  File "..\effects\shaders\rgb_to_yuv444.frag"
  File "..\effects\shaders\yuv444_to_rgb.frag"
  File "..\effects\shaders\yuv444_to_rgb.cg"
  File "..\effects\shaders\yuv444_to_rgb_deinterlace.cg"
  File "..\effects\shaders\yuv444_to_rgb_deinterlace.frag"
  File "..\effects\shaders\histogram.cg"
  File "..\effects\shaders\histogram.frag"
  File "..\effects\shaders\channel_selector.cg"
  File "..\effects\shaders\channel_selector.frag"

  SetDetailsPrint both

  ;--------------------------------
  ;Path environment variable

  Push "$INSTDIR\bin"
  Call AddToPath

  ;--------------------------------
  ;Include environment variable
  Push "OPENLIBRARIES_INCLUDE_PATH"
  Push "$INSTDIR\include\openlibraries-${VERSION_DOT}"
  Call WriteEnvStr

  ;--------------------------------
  ;Lib environment variable
  Push "OPENLIBRARIES_LIB_PATH"
  Push "$INSTDIR\lib"
  Call WriteEnvStr

  ;--------------------------------
  ;Python environment variable
  Push "PYTHONPATH"
  Push "$INSTDIR\python\Debug"
  Call WriteEnvStr

  ;--------------------------------
  ;Registry

  WriteRegStr HKLM "Software\OpenLibraries" "CurrentVersion"		"${VERSION_DOT}"
  WriteRegStr HKLM "Software\OpenLibraries" "Installation type"		"SDK"
  WriteRegStr HKLM "Software\OpenLibraries" "PluginsDirDebug"		"$INSTDIR\plugins\debug"
  WriteRegStr HKLM "Software\OpenLibraries" "PluginsDirRelease"		"$INSTDIR\plugins\release"
  WriteRegStr HKLM "Software\OpenLibraries" "KernelsDir"			"$INSTDIR\kernels"
  WriteRegStr HKLM "Software\OpenLibraries" "ShadersDir"			"$INSTDIR\shaders"

  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\OpenLibraries" "DisplayName" "OpenLibraries"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\OpenLibraries" "UninstallString" "$INSTDIR\uninst-openlibraries.exe"
  
SectionEnd

Section -post

	WriteUninstaller $INSTDIR\uninst-openlibraries.exe

SectionEnd

;--------------------------------
;Descriptions

;!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
;  !insertmacro MUI_DESCRIPTION_TEXT ${openal} "OpenAL"
;  !insertmacro MUI_DESCRIPTION_TEXT ${SecCore} "OpenLibraries development and runtime files"
;!insertmacro MUI_FUNCTION_DESCRIPTION_END

;--------------------------------
;Uninstaller Section

Section Uninstall

  SetDetailsPrint textonly
  DetailPrint "Deleting Files..."
  SetDetailsPrint listonly

  DeleteRegKey HKLM "Software\OpenLibraries"
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\OpenLibraries"

  Push "$INSTDIR\bin"
  Call un.RemoveFromPath

  Push "OPENLIBRARIES_LIB_PATH"
  Call un.DeleteEnvStr

  Push "OPENLIBRARIES_INCLUDE_PATH"
  Call un.DeleteEnvStr

  Push "PYTHONPATH"
  Call un.DeleteEnvStr

  RMDir /r $INSTDIR\bin
  RMDir /r $INSTDIR\include
  RMDir /r $INSTDIR\lib
  RMDir /r $INSTDIR\plugins
  RMDir /r $INSTDIR\shaders
  RMDir /r $INSTDIR

  SetDetailsPrint both
 
 SectionEnd
  
